;********************************************************
;
;              Decorative light Controller
;         for checking RB0 interrupt operation
;
;                                 Device : PIC16F628
;                                 Clock  : 4MHz
;                                 Author : Seiichi Inoue
;********************************************************
;When operation of RB0 interruption is normal, RA0 port
;will be made ON and OFF in the interval of 1 second in
;case of 50Hz. In case of 60Hz, it will be 0.83 seconds.

        list            p=pic16f628
        include         p16f628.inc
 __config _hs_osc & _wdt_off & _pwrte_on & _lvp_off & _boden_off
        errorlevel      -302    ;Eliminate bank warning

;**************  Operating condition  *******************

;*** Frequency of AC power. 50Hz or 60Hz
hz      equ    d'50'            ;Except 50Hz assume 60Hz

;****************  Label Definition  ********************
        cblock  h'20'
tmr0_value                      ;Value set to TMR0
tmr0_index                      ;TMR0 index
porta_work                      ;PORTA work area
portb_work                      ;PORTB work area
triac1                          ;TRIAC1 data
triac2                          ;TRIAC2 data
triac3                          ;TRIAC3 data
triac4                          ;TRIAC4 data
triac5                          ;TRIAC5 data
sw1_mode                        ;BCD SW1 mode
sw2_mode                        ;BCD SW2 mode
sw3_mode                        ;BCD SW3 mode
sw4_mode                        ;BCD SW4 mode
sw5_mode                        ;BCD SW5 mode
sw_count                        ;BCD SW scan counter
p_count                         ;Pattern trigger counter

;TRIAC1 work area
t1_s                            ;Start delay
t1_r                            ;Rising up
t1_b                            ;Bright
t1_f                            ;Falling down
t1_d                            ;Dim
t1_proc                         ;Process No.
t1_count                        ;Counter
t1_flag                         ;Counter Flag
t1_index                        ;Index
t1_subindex                     ;Sub Index

;TRIAC2 work area
t2_s                            ;Start delay
t2_r                            ;Rising up
t2_b                            ;Bright
t2_f                            ;Falling down
t2_d                            ;Dim
t2_proc                         ;Process No.
t2_count                        ;Counter
t2_flag                         ;Counter Flag
t2_index                        ;Index
t2_subindex                     ;Sub Index

;TRIAC3 work area
t3_s                            ;Start delay
t3_r                            ;Rising up
t3_b                            ;Bright
t3_f                            ;Falling down
t3_d                            ;Dim
t3_proc                         ;Process No.
t3_count                        ;Counter
t3_flag                         ;Counter Flag
t3_index                        ;Index
t3_subindex                     ;Sub Index

;TRIAC4 work area
t4_s                            ;Start delay
t4_r                            ;Rising up
t4_b                            ;Bright
t4_f                            ;Falling down
t4_d                            ;Dim
t4_proc                         ;Process No.
t4_count                        ;Counter
t4_flag                         ;Counter Flag
t4_index                        ;Index
t4_subindex                     ;Sub Index

;TRIAC5 work area
t5_s                            ;Start delay
t5_r                            ;Rising up
t5_b                            ;Bright
t5_f                            ;Falling down
t5_d                            ;Dim
t5_proc                         ;Process No.
t5_count                        ;Counter
t5_flag                         ;Counter Flag
t5_index                        ;Index
t5_subindex                     ;Sub Index
        endc

ra0     equ     00              ;RA0 bit
ra1     equ     01              ;RA1 bit
ra2     equ     02              ;RA2 bit
ra3     equ     03              ;RA3 bit
ra4     equ     04              ;RA4 bit


;***************** Program Start ************************
        org     0               ;Reset Vector
        goto    init
        org     4               ;Interrupt Vector
        goto    int

;******************* Initial Process ********************
init
;*** Set Port mode
        bsf     status,rp0      ;Change to Bank1
        movlw   b'00000000'     ;RA4-0:OUT mode
        movwf   trisa           ;Set TRISA reg
        movlw   b'00011111'     ;RB4-0:IN RB7-5:OUT mode
        movwf   trisb           ;Set TRISB reg

;*** Set Option reg
        movlw   b'01000001'     ;RBPU=ON,INTEDG=ON,PS=1:4
        movwf   option_reg      ;Set OPTION_REG

;*** Set PIE1 reg
        movlw   b'00000000'     ;TMR1IE=ON(Stop)
        movwf   pie1            ;Set PIE1 reg
        bcf     status,rp0      ;Change to Bank0

;*** Set CMCON reg
        movlw   b'00000111'     ;RA port to digital
        movwf   cmcon           ;Set CMCON reg

;*** Set T1CON reg
        movlw   b'00000001'     ;T1CKPS=1:1,TMR1=ON
        movwf   t1con           ;Set T1CON reg

;*** Set TMR0 reg
        movlw   d'50'           ;Set check data
        sublw   hz              ;Check frequency
        btfss   status,z        ;50Hz ?
        goto    hz60            ;No.
        movlw   d'9'            ;256-1ms/0.004ms=6+Adj
        goto    tmr0set
hz60
        movlw   d'51'           ;256-833us/4us=48+Adj
tmr0set
        movwf   tmr0_value      ;Save TMR0 value
        movwf   tmr0            ;Set TMR0

;*** Set TMR1H,TMR1L reg (50ms)
        movlw   h'3c'           ;65536-50000us=15536+Adj
        movwf   tmr1h           ;d'15550' = h'3CBE'
        movlw   h'be'           ;Set TMR1H = h'3C'
        movwf   tmr1l           ;Set TMR1L = h'BE'

;*** Other registers and work area initialization
        clrf    porta           ;Clear PORTA
        clrf    portb           ;Clear PORTB
        clrf    tmr0_index      ;Clear TMR0 index
        clrf    porta_work      ;Clear PORTA work area
        movlw   d'10'           ;Set TRIAC OFF data
        movwf   triac1          ;Set TRIAC1 data
        movwf   triac2          ;Set TRIAC2 data
        movwf   triac3          ;Set TRIAC3 data
        movwf   triac4          ;Set TRIAC4 data
        movwf   triac5          ;Set TRIAC5 data
        movlw   d'100'          ;Set value(For test)
        movwf   p_count         ;Set pattten trigger cnt
        clrf    sw_count        ;Clear switch scan count
        clrf    sw1_mode        ;Clear switch mode TRIAC1
        clrf    sw2_mode        ;Clear switch mode TRIAC2
        clrf    sw3_mode        ;Clear switch mode TRIAC3
        clrf    sw4_mode        ;Clear switch mode TRIAC4
        clrf    sw5_mode        ;Clear switch mode TRIAC5
        clrf    t1_proc         ;Clear process for TRIAC1
        clrf    t2_proc         ;Clear process for TRIAC2
        clrf    t3_proc         ;Clear process for TRIAC3
        clrf    t4_proc         ;Clear process for TRIAC4
        clrf    t5_proc         ;Clear process for TRIAC5

;*** Set interrupt condition
        movlw   b'11010000'     ;GIE,PEIE,(TOIE),INTE=1
        movwf   intcon          ;Interruption enable

        goto    $               ;Wait interruption

int
;*********** Interruption process beginning *************
        btfsc   intcon,intf     ;RB0 interruption ?
        goto    rb0_int         ;Jump to RB0 interruption
        btfsc   intcon,t0if     ;TMR0 time out ?
        goto    tmr0_int        ;Jump to TMR0 time-out
        btfsc   pir1,tmr1if     ;TMR1 time out ?
        goto    tmr1_int        ;Jump to TMR1 time-out
        goto    init            ;Reset(Illegal interrupt)

int_end
;************* END of interruption process **************
        retfie

rb0_int
;*************** RB0 interruption process ***************
        bcf     intcon,intf     ;Clear RB0 int flag
        bcf     intcon,t0if     ;Clear TMR0 int flag
        movfw   tmr0_value      ;Set timer value
        movwf   tmr0            ;Set TMR0
;        clrf    tmr0_index      ;Clear TMR0 index
;        clrf    porta_work      ;Clear PORTA work area
;        clrf    porta           ;Clear PORTA

;*** RB0 interruption check process
        decfsz  p_count,f       ;Count out ?(1sec)
        goto    int_end         ;No
        movlw   d'100'          ;Yes, Set count value
        movwf   p_count         ;Save count value
        movfw   porta_work      ;Read Flag
        btfss   status,z        ;RA0 OFF ?
        goto    rb0_1           ;No
        movlw   b'00000001'     ;Yes
        movwf   porta_work      ;Save Flag
        movwf   porta           ;RA0 ON
        goto    int_end
rb0_1   clrf    porta_work      ;Clear Flag
        clrf    porta           ;RA0 OFF
;*** End of RB0 interruption check process

        goto    int_end         ;End of RB0 interruption

tmr0_int
;************** TMR0 interruption process ***************
        bcf     intcon,t0if     ;Clear TMR0 int flag
        movfw   tmr0_value      ;Set timer value
        movwf   tmr0            ;Set TMR0
        goto    int_end         ;(Add)End of TMR0

tmr1_int
;***** Timer1 interruption process (50ms interval) ******
        bcf     pir1,tmr1if     ;Clear TMR1 int flag
        movlw   h'3c'           ;Set TMR1H value
        movwf   tmr1h           ;Set TMR1H
        movlw   h'be'           ;Set TMR1L value
        movwf   tmr1l           ;Set TMR1L
        goto    int_end         ;(Add)End of TMR0


;********************************************************
;          END of Decorative light Controller
;********************************************************

        end
